@@ -1,5 +1,5 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #pragma once
 
 #include "cmConfigure.h" // IWYU pragma: keep
@@ -23,7 +23,6 @@
 #include "cmGlobalCommonGenerator.h"
 #include "cmGlobalGeneratorFactory.h"
 #include "cmNinjaTypes.h"
-#include "cmPolicies.h"
 #include "cmStringAlgorithms.h"
 #include "cmTransformDepfile.h"
 
@@ -58,14 +57,14 @@ class cmGlobalNinjaGenerator : public cmGlobalCommonGenerator
 {
 public:
   /// The default name of Ninja's build file. Typically: build.ninja.
-  static const char* NINJA_BUILD_FILE;
+  static char const* NINJA_BUILD_FILE;
 
   /// The default name of Ninja's rules file. Typically: rules.ninja.
   /// It is included in the main build.ninja file.
-  static const char* NINJA_RULES_FILE;
+  static char const* NINJA_RULES_FILE;
 
   /// The indentation string used when generating Ninja's build file.
-  static const char* INDENT;
+  static char const* INDENT;
 
   /// The shell command used for a no-op.
   static std::string const SHELL_NOOP;
@@ -78,8 +77,8 @@ public:
 
   static std::string EncodeRuleName(std::string const& name);
   std::string& EncodeLiteral(std::string& lit) override;
-  std::string GetEncodedLiteral(const std::string& lit);
-  std::string EncodePath(const std::string& path);
+  std::string GetEncodedLiteral(std::string const& lit);
+  std::string EncodePath(std::string const& path);
 
   std::unique_ptr<cmLinkLineComputer> CreateLinkLineComputer(
     cmOutputConverter* outputConverter,
@@ -89,7 +88,7 @@ public:
    * Write the given @a comment to the output stream @a os. It
    * handles new line character properly.
    */
-  static void WriteComment(std::ostream& os, const std::string& comment);
+  static void WriteComment(std::ostream& os, std::string const& comment);
 
   /**
    * Utilized by the generator factory to determine if this generator
@@ -137,7 +136,7 @@ public:
                                cmNinjaDeps orderOnlyDeps = cmNinjaDeps());
 
   void WriteMacOSXContentBuild(std::string input, std::string output,
-                               const std::string& config);
+                               std::string const& config);
 
   /**
    * Write a rule statement to @a os.
@@ -150,23 +149,23 @@ public:
    * optional @a comment. An @a indent level can be specified.
    * @warning no escaping of any kind is done here.
    */
-  static void WriteVariable(std::ostream& os, const std::string& name,
-                            const std::string& value,
-                            const std::string& comment = "", int indent = 0);
+  static void WriteVariable(std::ostream& os, std::string const& name,
+                            std::string const& value,
+                            std::string const& comment = "", int indent = 0);
 
   /**
    * Write an include statement including @a filename with an optional
    * @a comment to the @a os stream.
    */
-  static void WriteInclude(std::ostream& os, const std::string& filename,
-                           const std::string& comment = "");
+  static void WriteInclude(std::ostream& os, std::string const& filename,
+                           std::string const& comment = "");
 
   /**
    * Write a default target statement specifying @a targets as
    * the default targets.
    */
-  static void WriteDefault(std::ostream& os, const cmNinjaDeps& targets,
-                           const std::string& comment = "");
+  static void WriteDefault(std::ostream& os, cmNinjaDeps const& targets,
+                           std::string const& comment = "");
 
   bool IsGCCOnWindows() const { return this->UsingGCCOnWindows; }
   void MarkAsGCCOnWindows() { this->UsingGCCOnWindows = true; }
@@ -200,39 +199,39 @@ public:
                       cmMakefile* mf, bool optional) override;
 
   std::vector<GeneratedMakeCommand> GenerateBuildCommand(
-    const std::string& makeProgram, const std::string& projectName,
-    const std::string& projectDir, std::vector<std::string> const& targetNames,
-    const std::string& config, int jobs, bool verbose,
-    const cmBuildOptions& buildOptions = cmBuildOptions(),
+    std::string const& makeProgram, std::string const& projectName,
+    std::string const& projectDir, std::vector<std::string> const& targetNames,
+    std::string const& config, int jobs, bool verbose,
+    cmBuildOptions const& buildOptions = cmBuildOptions(),
     std::vector<std::string> const& makeOptions =
       std::vector<std::string>()) override;
 
   // Setup target names
-  const char* GetAllTargetName() const override { return "all"; }
-  const char* GetInstallTargetName() const override { return "install"; }
-  const char* GetInstallLocalTargetName() const override
+  char const* GetAllTargetName() const override { return "all"; }
+  char const* GetInstallTargetName() const override { return "install"; }
+  char const* GetInstallLocalTargetName() const override
   {
     return "install/local";
   }
-  const char* GetInstallStripTargetName() const override
+  char const* GetInstallStripTargetName() const override
   {
     return "install/strip";
   }
-  const char* GetInstallParallelTargetName() const
+  char const* GetInstallParallelTargetName() const
   {
     return "install/parallel";
   }
-  const char* GetTestTargetName() const override { return "test"; }
-  const char* GetPackageTargetName() const override { return "package"; }
-  const char* GetPackageSourceTargetName() const override
+  char const* GetTestTargetName() const override { return "test"; }
+  char const* GetPackageTargetName() const override { return "package"; }
+  char const* GetPackageSourceTargetName() const override
   {
     return "package_source";
   }
-  const char* GetRebuildCacheTargetName() const override
+  char const* GetRebuildCacheTargetName() const override
   {
     return "rebuild_cache";
   }
-  const char* GetCleanTargetName() const override { return "clean"; }
+  char const* GetCleanTargetName() const override { return "clean"; }
 
   bool SupportsCustomCommandDepfile() const override { return true; }
   cm::optional<cmDepfileFormat> DepfileFormat() const override
@@ -243,13 +242,13 @@ public:
   bool SupportsLinkerDependencyFile() const override { return true; }
 
   virtual cmGeneratedFileStream* GetImplFileStream(
-    const std::string& /*config*/) const
+    std::string const& /*config*/) const
   {
     return this->BuildFileStream.get();
   }
 
   virtual cmGeneratedFileStream* GetConfigFileStream(
-    const std::string& /*config*/) const
+    std::string const& /*config*/) const
   {
     return this->BuildFileStream.get();
   }
@@ -269,7 +268,7 @@ public:
     return this->RulesFileStream.get();
   }
 
-  std::string const& ConvertToNinjaPath(const std::string& path) const;
+  std::string const& ConvertToNinjaPath(std::string const& path) const;
   std::string ConvertToNinjaAbsPath(std::string path) const;
 
   struct MapToNinjaPathImpl
@@ -291,20 +290,20 @@ public:
 #endif
 
   // -- Additional clean files
-  void AddAdditionalCleanFile(std::string fileName, const std::string& config);
-  const char* GetAdditionalCleanTargetName() const
+  void AddAdditionalCleanFile(std::string fileName, std::string const& config);
+  char const* GetAdditionalCleanTargetName() const
   {
     return "CMakeFiles/clean.additional";
   }
 
-  static const char* GetByproductsForCleanTargetName()
+  static char const* GetByproductsForCleanTargetName()
   {
     return "CMakeFiles/cmake_byproducts_for_clean_target";
   }
 
-  void AddCXXCompileCommand(const std::string& commandLine,
-                            const std::string& sourceFile,
-                            const std::string& objPath);
+  void AddCXXCompileCommand(std::string const& commandLine,
+                            std::string const& sourceFile,
+                            std::string const& objPath);
 
   /**
    * Add a rule to the generated build system.
@@ -313,12 +312,12 @@ public:
    */
   void AddRule(cmNinjaRule const& rule);
 
-  bool HasRule(const std::string& name);
+  bool HasRule(std::string const& name);
 
   void AddCustomCommandRule();
   void AddMacOSXContentRule();
 
-  bool HasCustomCommandOutput(const std::string& output)
+  bool HasCustomCommandOutput(std::string const& output)
   {
     return this->CustomCommandOutputs.find(output) !=
       this->CustomCommandOutputs.end();
@@ -326,13 +325,13 @@ public:
 
   /// Called when we have seen the given custom command.  Returns true
   /// if we has seen it before.
-  bool SeenCustomCommand(cmCustomCommand const* cc, const std::string& config)
+  bool SeenCustomCommand(cmCustomCommand const* cc, std::string const& config)
   {
     return !this->Configs[config].CustomCommands.insert(cc).second;
   }
 
   /// Called when we have seen the given custom command output.
-  void SeenCustomCommandOutput(const std::string& output)
+  void SeenCustomCommandOutput(std::string const& output)
   {
     this->CustomCommandOutputs.insert(output);
     // We don't need the assumed dependencies anymore, because we have
@@ -340,8 +339,8 @@ public:
     this->AssumedSourceDependencies.erase(output);
   }
 
-  void AddAssumedSourceDependencies(const std::string& source,
-                                    const cmNinjaDeps& deps)
+  void AddAssumedSourceDependencies(std::string const& source,
+                                    cmNinjaDeps const& deps)
   {
     std::set<std::string>& ASD = this->AssumedSourceDependencies[source];
     // Because we may see the same source file multiple times (same source
@@ -351,46 +350,47 @@ public:
   }
 
   virtual std::string OrderDependsTargetForTarget(
-    cmGeneratorTarget const* target, const std::string& config) const;
+    cmGeneratorTarget const* target, std::string const& config) const;
 
   std::string OrderDependsTargetForTargetPrivate(
-    cmGeneratorTarget const* target, const std::string& config) const;
+    cmGeneratorTarget const* target, std::string const& config) const;
 
   void AppendTargetOutputs(cmGeneratorTarget const* target,
-                           cmNinjaDeps& outputs, const std::string& config,
+                           cmNinjaDeps& outputs, std::string const& config,
                            cmNinjaTargetDepends depends) const;
   void AppendTargetDepends(cmGeneratorTarget const* target,
-                           cmNinjaDeps& outputs, const std::string& config,
-                           const std::string& fileConfig,
+                           cmNinjaDeps& outputs, std::string const& config,
+                           std::string const& fileConfig,
                            cmNinjaTargetDepends depends);
   void AppendTargetDependsClosure(cmGeneratorTarget const* target,
                                   std::unordered_set<std::string>& outputs,
-                                  const std::string& config,
-                                  const std::string& fileConfig,
+                                  std::string const& config,
+                                  std::string const& fileConfig,
                                   bool genexOutput, bool omit_self = true);
 
-  void AppendDirectoryForConfig(const std::string& prefix,
-                                const std::string& config,
-                                const std::string& suffix,
+  void AppendDirectoryForConfig(std::string const& prefix,
+                                std::string const& config,
+                                std::string const& suffix,
                                 std::string& dir) override;
 
   virtual void AppendNinjaFileArgument(GeneratedMakeCommand& /*command*/,
-                                       const std::string& /*config*/) const
+                                       std::string const& /*config*/) const
   {
   }
 
   virtual void AddRebuildManifestOutputs(cmNinjaDeps& outputs) const
   {
     outputs.push_back(this->NinjaOutputPath(NINJA_BUILD_FILE));
+    this->AddCMakeFilesToRebuild(outputs);
   }
 
-  int GetRuleCmdLength(const std::string& name)
+  int GetRuleCmdLength(std::string const& name)
   {
     return this->RuleCmdLength[name];
   }
 
-  void AddTargetAlias(const std::string& alias, cmGeneratorTarget* target,
-                      const std::string& config);
+  void AddTargetAlias(std::string const& alias, cmGeneratorTarget* target,
+                      std::string const& config);
 
   void ComputeTargetObjectDirectory(cmGeneratorTarget* gt) const override;
 
@@ -440,13 +440,13 @@ public:
     std::string const& arg_lang, std::string const& arg_modmapfmt,
     cmCxxModuleExportInfo const& export_info);
 
-  virtual std::string BuildAlias(const std::string& alias,
-                                 const std::string& /*config*/) const
+  virtual std::string BuildAlias(std::string const& alias,
+                                 std::string const& /*config*/) const
   {
     return alias;
   }
 
-  virtual std::string ConfigDirectory(const std::string& /*config*/) const
+  virtual std::string ConfigDirectory(std::string const& /*config*/) const
   {
     return "";
   }
@@ -456,26 +456,26 @@ public:
     return this->ByproductsForCleanTarget;
   }
 
-  cmNinjaDeps& GetByproductsForCleanTarget(const std::string& config)
+  cmNinjaDeps& GetByproductsForCleanTarget(std::string const& config)
   {
     return this->Configs[config].ByproductsForCleanTarget;
   }
 
   bool EnableCrossConfigBuild() const;
 
-  std::set<std::string> GetCrossConfigs(const std::string& config) const;
+  std::set<std::string> GetCrossConfigs(std::string const& config) const;
 
-  const std::set<std::string>& GetDefaultConfigs() const
+  std::set<std::string> const& GetDefaultConfigs() const override
   {
     return this->DefaultConfigs;
   }
 
-  const std::set<std::string>& GetPerConfigUtilityTargets() const
+  std::set<std::string> const& GetPerConfigUtilityTargets() const
   {
     return this->PerConfigUtilityTargets;
   }
 
-  void AddPerConfigUtilityTarget(const std::string& name)
+  void AddPerConfigUtilityTarget(std::string const& name)
   {
     this->PerConfigUtilityTargets.insert(name);
   }
@@ -483,6 +483,9 @@ public:
   bool IsSingleConfigUtility(cmGeneratorTarget const* target) const;
 
   bool CheckCxxModuleSupport(CxxModuleSupportQuery query) override;
+  bool SupportsBuildDatabase() const override { return true; }
+
+  std::string ConvertToOutputPath(std::string path) const override;
 
 protected:
   std::vector<std::string> const& GetConfigNames() const;
@@ -495,11 +498,11 @@ protected:
   virtual void CloseBuildFileStreams();
 
   bool OpenFileStream(std::unique_ptr<cmGeneratedFileStream>& stream,
-                      const std::string& name);
+                      std::string const& name);
 
   static cm::optional<std::set<std::string>> ListSubsetWithAll(
-    const std::set<std::string>& all, const std::set<std::string>& defaults,
-    const std::vector<std::string>& items);
+    std::set<std::string> const& all, std::set<std::string> const& defaults,
+    std::vector<std::string> const& items);
 
   std::set<std::string> CrossConfigs;
   std::set<std::string> DefaultConfigs;
@@ -527,13 +530,16 @@ private:
 
   void WriteTargetAliases(std::ostream& os);
   void WriteFolderTargets(std::ostream& os);
-  void WriteUnknownExplicitDependencies(std::ostream& os);
 
   void WriteBuiltinTargets(std::ostream& os);
   void WriteTargetDefault(std::ostream& os);
   void WriteTargetRebuildManifest(std::ostream& os);
   bool WriteTargetCleanAdditional(std::ostream& os);
   void WriteTargetClean(std::ostream& os);
+#if !defined(CMAKE_BOOTSTRAP) && !defined(_WIN32)
+  // FIXME(#26668) This does not work on Windows
+  void WriteTargetInstrument(std::ostream& os);
+#endif
   void WriteTargetHelp(std::ostream& os);
 
   void ComputeTargetDependsClosure(
@@ -562,18 +568,6 @@ private:
   /// The set of custom command outputs we have seen.
   std::set<std::string> CustomCommandOutputs;
 
-  /// Whether we are collecting known build outputs and needed
-  /// dependencies to determine unknown dependencies.
-  bool ComputingUnknownDependencies = false;
-  cmPolicies::PolicyStatus PolicyCMP0058 = cmPolicies::WARN;
-
-  /// The combined explicit dependencies of custom build commands
-  std::set<std::string> CombinedCustomCommandExplicitDependencies;
-
-  /// When combined with CombinedCustomCommandExplicitDependencies it allows
-  /// us to detect the set of explicit dependencies that have
-  std::set<std::string> CombinedBuildOutputs;
-
   /// The mapping from source file to assumed dependencies.
   std::map<std::string, std::set<std::string>> AssumedSourceDependencies;
 
@@ -647,27 +641,27 @@ private:
 
   cmNinjaDeps ByproductsForCleanTarget;
 
-  friend bool operator==(const ByConfig::TargetDependsClosureKey& lhs,
-                         const ByConfig::TargetDependsClosureKey& rhs);
-  friend bool operator!=(const ByConfig::TargetDependsClosureKey& lhs,
-                         const ByConfig::TargetDependsClosureKey& rhs);
-  friend bool operator<(const ByConfig::TargetDependsClosureKey& lhs,
-                        const ByConfig::TargetDependsClosureKey& rhs);
-  friend bool operator>(const ByConfig::TargetDependsClosureKey& lhs,
-                        const ByConfig::TargetDependsClosureKey& rhs);
-  friend bool operator<=(const ByConfig::TargetDependsClosureKey& lhs,
-                         const ByConfig::TargetDependsClosureKey& rhs);
-  friend bool operator>=(const ByConfig::TargetDependsClosureKey& lhs,
-                         const ByConfig::TargetDependsClosureKey& rhs);
+  friend bool operator==(ByConfig::TargetDependsClosureKey const& lhs,
+                         ByConfig::TargetDependsClosureKey const& rhs);
+  friend bool operator!=(ByConfig::TargetDependsClosureKey const& lhs,
+                         ByConfig::TargetDependsClosureKey const& rhs);
+  friend bool operator<(ByConfig::TargetDependsClosureKey const& lhs,
+                        ByConfig::TargetDependsClosureKey const& rhs);
+  friend bool operator>(ByConfig::TargetDependsClosureKey const& lhs,
+                        ByConfig::TargetDependsClosureKey const& rhs);
+  friend bool operator<=(ByConfig::TargetDependsClosureKey const& lhs,
+                         ByConfig::TargetDependsClosureKey const& rhs);
+  friend bool operator>=(ByConfig::TargetDependsClosureKey const& lhs,
+                         ByConfig::TargetDependsClosureKey const& rhs);
 };
 
 class cmGlobalNinjaMultiGenerator : public cmGlobalNinjaGenerator
 {
 public:
   /// The default name of Ninja's common file. Typically: common.ninja.
-  static const char* NINJA_COMMON_FILE;
+  static char const* NINJA_COMMON_FILE;
   /// The default file extension to use for per-config Ninja files.
-  static const char* NINJA_FILE_EXTENSION;
+  static char const* NINJA_FILE_EXTENSION;
 
   cmGlobalNinjaMultiGenerator(cmake* cm);
   bool IsMultiConfig() const override { return true; }
@@ -686,8 +680,8 @@ public:
 
   static std::string GetActualName() { return "Ninja Multi-Config"; }
 
-  std::string BuildAlias(const std::string& alias,
-                         const std::string& config) const override
+  std::string BuildAlias(std::string const& alias,
+                         std::string const& config) const override
   {
     if (config.empty()) {
       return alias;
@@ -695,7 +689,7 @@ public:
     return cmStrCat(alias, ":", config);
   }
 
-  std::string ConfigDirectory(const std::string& config) const override
+  std::string ConfigDirectory(std::string const& config) const override
   {
     if (!config.empty()) {
       return cmStrCat('/', config);
@@ -703,19 +697,19 @@ public:
     return "";
   }
 
-  const char* GetCMakeCFGIntDir() const override { return "${CONFIGURATION}"; }
+  char const* GetCMakeCFGIntDir() const override { return "${CONFIGURATION}"; }
 
-  std::string ExpandCFGIntDir(const std::string& str,
-                              const std::string& config) const override;
+  std::string ExpandCFGIntDir(std::string const& str,
+                              std::string const& config) const override;
 
   cmGeneratedFileStream* GetImplFileStream(
-    const std::string& config) const override
+    std::string const& config) const override
   {
     return this->ImplFileStreams.at(config).get();
   }
 
   cmGeneratedFileStream* GetConfigFileStream(
-    const std::string& config) const override
+    std::string const& config) const override
   {
     return this->ConfigFileStreams.at(config).get();
   }
@@ -731,10 +725,10 @@ public:
   }
 
   void AppendNinjaFileArgument(GeneratedMakeCommand& command,
-                               const std::string& config) const override;
+                               std::string const& config) const override;
 
-  static std::string GetNinjaImplFilename(const std::string& config);
-  static std::string GetNinjaConfigFilename(const std::string& config);
+  static std::string GetNinjaImplFilename(std::string const& config);
+  static std::string GetNinjaConfigFilename(std::string const& config);
 
   void AddRebuildManifestOutputs(cmNinjaDeps& outputs) const override;
 
@@ -749,7 +743,7 @@ public:
   bool SupportsDefaultConfigs() const override { return true; }
 
   std::string OrderDependsTargetForTarget(
-    cmGeneratorTarget const* target, const std::string& config) const override;
+    cmGeneratorTarget const* target, std::string const& config) const override;
 
 protected:
   bool OpenBuildFileStreams() override;
